# ============================================================================
# Replicating Tsai & Xu (2018) Tutorial 
# ============================================================================

# ==== Load Required Packages ====
packages <- c("haven", "dplyr", "fixest", "modelsummary", "ggplot2", 
              "sensemakr", "tidyr", "estimatr", "purrr", "broom", "patchwork",
              "Matching")

for (pkg in packages) {
  if (!requireNamespace(pkg, quietly = TRUE)) {
    install.packages(pkg)
  }
  library(pkg, character.only = TRUE)
}

# ------------------------------------------------------------------------------
#                               DATA IMPORT & SETUP
# ------------------------------------------------------------------------------
urban         <- read_dta("insider_urban.dta")   %>% mutate(distrid = factor(distrid))
rural         <- read_dta("insider_rural.dta")   %>% mutate(v_id    = factor(v_id))

par(mfrow = c(1,2))
issue_counts <- urban %>%
  filter(!is.na(issue)) %>%  # This line drops NA values
  count(issue) %>%
  mutate(issue_label = case_when(
    issue == 1 ~ "Public Security",
    issue == 2 ~ "Transportation",
    issue == 3 ~ "Utilities",
    issue == 4 ~ "Food and Drug Safety",
    issue == 5 ~ "Air Pollution",
    issue == 6 ~ "Community Environment",
    issue == 7 ~ "Public Health",
    issue == 8 ~ "Licenses, Permits, and Certificates",
    issue == 9 ~ "Right infringement",
    issue == 10 ~ "Misc.",
    TRUE ~ "Unknown"
  )) %>%
  mutate(issue_label = factor(issue_label, 
  levels = c("Food and Drug Safety", "Transportation", "Public Security", "Utilities", "Community Environment", "Right infringement", "Licenses, Permits, and Certificates", "Air Pollution", "Public Health", "Misc.")))

## ------------------------------------------------------------------
## Descriptives
## ------------------------------------------------------------------

p1 <- issue_counts |>
  ggplot(aes(issue_label, n)) +
  geom_bar(stat = "identity", fill = "steelblue") +
  labs(x = NULL, y = "Number of Complaints",
       subtitle = "(a) Issues", caption = "") +
  scale_y_continuous(expand = expansion(mult = c(0, 0.1))) +
  theme_minimal() +
  theme(axis.text.x   = element_text(angle = 45, hjust = 1),
        plot.subtitle = element_text(hjust = .5))


concern_data <- urban %>%
  filter(!is.na(concernme), !is.na(govconn)) %>%
  mutate(concern_cat = case_when(
    concernme == 1 ~ "Only concerns me",
    concernme == 2 ~ "Concerns many people",
    concernme == 3 ~ "Concerns almost everybody"
  ),
  govconn_group = ifelse(govconn == 1, "Insider", "Outsider"))

# Calculate percentages
concern_summary <- concern_data %>%
  group_by(govconn_group, concern_cat) %>%
  summarise(count = n()) %>%
  group_by(govconn_group) %>%
  mutate(percentage = count / sum(count) * 100)

p2 <- concern_summary |>
  ggplot(aes(concern_cat, percentage, fill = govconn_group)) +
  geom_bar(stat = "identity", position = "dodge") +
  geom_text(aes(label = paste0(round(percentage), "%")),
            position = position_dodge(width = .9),
            vjust = -.5, size = 3) +
  labs(x = NULL, y = "Percentage",
       subtitle = "(b) Civic versus Private Interests",
       fill = NULL) +
  scale_fill_manual(values = c(Insider = "orange", Outsider = "steelblue")) +
  scale_y_continuous(limits = c(0, 50), expand = expansion(mult = c(0, 0.1))) +
  theme_minimal() +
  theme(axis.text.x   = element_text(angle = 45, hjust = 1),
        plot.subtitle = element_text(hjust = .5),
        legend.position = "top")

p1 + p2 + plot_layout(ncol = 2)

covar1    <- c("eduyr","ccp","govoff","age","age2","male","hukou")
covar2    <- c("eduyr","ccp","leader","age","age2","male")


# ------------------------------------------------------------------------------
#                               MAIN RESULTS
# ------------------------------------------------------------------------------

##################################################################
## Figure: Table 1 & 2
##################################################################
covar1    <- c("eduyr","ccp","govoff","age","age2","male","hukou")
covar2    <- c("eduyr","ccp","leader","age","age2","male")
# Ubran 
## helper
fmla <- function(dv, with_ctrl = FALSE) {
  rhs <- if (with_ctrl) c("govconn", covar1) else "govconn"
  reformulate(rhs, dv)
}

fit <- function(formula) lm_robust(
  formula,
  data          = urban,
  fixed_effects = ~ distrid,
  clusters      = distrid
)

## models
mods_u <- list(
  "To government"                           = fit(fmla("compgov")),
  "To government w controls"                = fit(fmla("compgov", TRUE)),
  "Through government offices"              = fit(fmla("comp_off")),
  "Through government offices w controls"   = fit(fmla("comp_off", TRUE))
)

## clean names 
coef_map <- c(
  govconn = "Political connections",
  eduyr   = "Years of education",
  ccp     = "Communist party member",
  govoff  = "Government official",
  age     = "Age/10",
  age2    = "Age²/100",
  male    = "Male",
  hukou   = "Urban Hukou"
)

# Rural 
## helper function to create formulas
fmla <- function(dv, with_ctrl = FALSE) {
  rhs <- if (with_ctrl) c("govconn", covar2) else "govconn"
  reformulate(rhs, dv)
}

fit <- function(formula) {
  lm_robust(
    formula,
    data = rural, 
    fixed_effects = ~ v_id, 
    clusters = v_id
  )
}

mods_r <- list(
  "To village committee" = fit(fmla("compl")),
  "To village committee w controls" = fit(fmla("compl", TRUE)),
  "To fellow villagers" = fit(fmla("compl_vill")), 
  "To fellow villagers w controls" = fit(fmla("compl_vill", TRUE))
)


coef_map <- c(
  govconn = "Political connections (govconn)", # Clarified this is the IV
  eduyr = "Years of education",
  ccp = "Communist party member",
  leader = "Village leader", # Changed from Government official
  age = "Age/10",
  age2 = "Age²/100",
  male = "Male"
)

# Create a dataframe for plotting coefficients of 'govconn'
coef_df_r <- bind_rows(
  lapply(names(mods_r), \(nm)
         tidy(mods_r[[nm]], conf.int = TRUE) |>
           filter(term == "govconn") |> # IV is govconn
           mutate(model = nm))
)


# collect govconn coefficients only for models with controls 
grab_coef <- function(model_list, area_label){
  bind_rows(lapply(names(model_list), \(nm){
    tidy(model_list[[nm]], conf.int = TRUE) |>
      filter(term == "govconn") |>
      mutate(outcome = nm,
             area    = area_label)
  }))
}
coef_df <- bind_rows(
  grab_coef(mods_u, "Urban"),
  grab_coef(mods_r, "Rural")
) |>
  filter(grepl("w controls", outcome)) |>                 # keep only control specs
  mutate(outcome = sub(" w controls", "", outcome),       # shorten labels
         outcome = factor(outcome, levels = unique(outcome)))

coef_df <- coef_df |>
  mutate(area = factor(area, levels = c("Urban", "Rural")))
ggplot(coef_df, 
       aes(x = outcome,
           y = estimate,
           ymin = conf.low,
           ymax = conf.high)) +
  # Map linetype to area so the error‐bar shows up in the legend
  geom_errorbar(aes(linetype = area), width = 0.10, linewidth = 0.6) +
  geom_point(aes(shape = area), size = 3) +
  geom_text(aes(
    label = sprintf("%.2f", estimate),
    y = ifelse(estimate >= 0, conf.high + 0.01, conf.low - 0.01)
  ),
  size = 3, show.legend = FALSE) +
  geom_hline(yintercept = 0, linetype = "dashed") +
  facet_wrap(~ area, nrow = 1, scales = "free_x") +
  
  # Define a single legend (no title) that combines shape and linetype
  scale_shape_manual(
    name   = NULL,
    values = c(Urban = 16, Rural = 17),
    breaks = c("Urban", "Rural")
  ) +
  scale_linetype_manual(
    name   = NULL,
    values = c(Urban = "solid", Rural = "solid"),
    breaks = c("Urban", "Rural")
  ) +
  
  labs(
    y     = "Effect of political connections",
    x     = NULL
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.spacing = unit(1, "lines"),
    axis.text.x   = element_text(angle = 45, hjust = 1),
  )

##################################################################
## Figures 2: Mechanisms
##################################################################

# ---------------------- single model -------------------------
fit_fe <- function(data, dv, ctrls, fe, clust, area, panel){
  rhs  <- paste(c("govconn", ctrls), collapse = " + ")
  frm  <- as.formula(paste0(dv, " ~ ", rhs, " | ", fe))
  
  mod  <- feols(frm, data = data, cluster = clust)
  
  res  <- tidy(mod, conf.int = TRUE) |>
    filter(term == "govconn")
  
  tibble(
    panel     = panel,
    area      = area,
    outcome   = dv,
    coef      = res$estimate,
    se        = res$std.error,
    CI_lower  = res$conf.low,
    CI_upper  = res$conf.high,
    N_clust   = length(unique(data[[clust]])),
  )
}

run_block <- function(dvs, data, ctrls, fe, clust, area, panel){
  map_dfr(dvs, ~ fit_fe(data, .x, ctrls, fe, clust, area, panel))
}

# ------------------- dependent variables ---------------------
urb_know <- c("govfile","mayor","email")
urb_acc  <- c("backdoor","dealgov","pullstring")
rur_know <- c("xiangzhang","governor","newspaper")
rur_acc  <- grep("^welcome", names(rural), value = TRUE)

# ------------------- run everything --------------------------
results <- bind_rows(
  run_block(urb_know, urban, covar1, "distrid", "distrid",
            "Urban", "Political knowledge"),
  run_block(urb_acc , urban, covar1, "distrid", "distrid",
            "Urban", "Access to authorities"),
  run_block(rur_know, rural, covar2, "v_id",    "v_id",
            "Rural", "Political knowledge"),
  run_block(rur_acc , rural, covar2, "v_id",    "v_id",
            "Rural", "Access to authorities")
)

keep   <- c("govfile","mayor","email",
            "xiangzhang","governor","newspaper",
            "backdoor","dealgov","pullstring",
            "welcome_cadre","welcome_town","welcome_coty")

mechanisms <- results |>
  filter(outcome %in% keep) |>
  # readable labels, panels, and display order
  mutate(
    outcome_lbl = recode(outcome,
                         govfile        = "Access Gov.\nStatues",
                         mayor          = "Know\nMajor's Name",
                         email          = "Use Email\nRegularly",
                         xiangzhang     = "Know Township\nHead",
                         governor       = "Know\nGovernor's Name",
                         newspaper      = "Read\nNewspapers",
                         backdoor       = "Use Conn.\nfor Pub. Sev.",
                         dealgov        = "Deal with the\nGovernment",
                         pullstring     = "Pull Strings\nfor Benefits",
                         welcome_cadre  = "Cadres Welcome\nInputs",
                         welcome_town   = "Township\nWelcomes Inputs",
                         welcome_coty   = "County Welcomes\nInputs"
    ),
    panel = if_else(outcome %in% c("govfile","mayor","email",
                                   "xiangzhang","governor","newspaper"),
                    "Political knowledge",
                    "Access to authorities"),
    CI_lower = coef - 1.96 * se,
    CI_upper = coef + 1.96 * se,
    label_x  = coef + if_else(coef >= 0, 0.022, -0.022),
    hjust    = if_else(coef >= 0, 0, 1)
  ) |>
  # set plotting order exactly as in the article
  mutate(outcome_lbl = factor(outcome_lbl,
                              levels = c("Access Gov.\nStatues","Know\nMajor's Name",
                                         "Use Email\nRegularly","Know Township\nHead",
                                         "Know\nGovernor's Name","Read\nNewspapers",
                                         "Use Conn.\nfor Pub. Sev.","Deal with the\nGovernment",
                                         "Pull Strings\nfor Benefits","Cadres Welcome\nInputs",
                                         "Township\nWelcomes Inputs","County Welcomes\nInputs"))
  )

# 2. plot ----------------------------------------------------
# make sure the facet order is: 1) knowledge, 2) access
mechanisms <- mechanisms %>%
  mutate(
    panel = factor(panel, levels = c("Political knowledge", "Access to authorities")),
    # Reverse factor levels to control drawing order
    area = factor(area, levels = c("Rural", "Urban"))
  ) %>%
  arrange(panel, area, outcome_lbl) %>%  # Now Rural plots first (underneath)
  mutate(outcome_lbl = factor(outcome_lbl, levels = unique(outcome_lbl)))

# Plot with modified aesthetics
ggplot(mechanisms, aes(x = coef, y = outcome_lbl)) +
  geom_vline(xintercept = 0, colour = "grey70", linewidth = .3) +
  # Error bars with explicit linetype mapping
  geom_errorbarh(
    aes(xmin = CI_lower, xmax = CI_upper, linetype = area),
    height = .18, linewidth = .6, show.legend = TRUE
  ) +
  # Points with explicit shape mapping
  geom_point(
    aes(shape = area),
    size = 3
  ) +
  geom_text(
    aes(label = sprintf("%.2f", coef)),
    position = position_nudge(y = 0.25),
    size = 3, show.legend = FALSE
  ) +
  facet_wrap(~ panel, ncol = 1, scales = "free_y") +
  coord_cartesian(xlim = c(-.10, .30)) +
  # Manual scales with Urban as first element
  scale_linetype_manual(
    values = c("Urban" = "solid", "Rural" = "solid"),
    breaks = c("Urban", "Rural")  # This controls legend order
  ) +
  scale_shape_manual(
    values = c("Urban" = 16, "Rural" = 17),
    breaks = c("Urban", "Rural")
  ) +
  labs(
    x = "Effect of political connections",
    y = NULL,
    title = "Political Connections: Knowledge and Access"
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.grid.major.y = element_blank(),
    axis.text.y = element_text(size = 9),
    strip.text = element_text(face = "bold", size = 13),
    legend.title = element_blank()
  )

# ------------------------------------------------------------------------------
#                               MAIN RESULTS
# ------------------------------------------------------------------------------

##################################################################
## Table 4: Matching 
##################################################################

library("Matching")

# Drop NAs
vars_u <- c("compgov", "comp_off", "govconn", covar1)
urban_cc <- urban[complete.cases(urban[, vars_u]), ]

vars_r <- c("compl", "compl_vill", "govconn", covar2)
rural_cc <- rural[complete.cases(rural[, vars_r]), ]

matching_analysis <- function(data, Y, treat, covar, cluster_var) {
  # Ensure the treatment variable is binary (0/1)
  data[[treat]] <- as.numeric(data[[treat]])
  
  # Perform matching
  m.out <- Match(
    Y = data[[Y]],
    Tr = data[[treat]],
    X = data[, covar],
    estimand = "ATT",  # Average Treatment Effect on the Treated
    M = 5,
    replace = FALSE,   # No replacement for more conservative estimates
    ties = TRUE,
    BiasAdjust = TRUE
  )
  
  # Calculate clustered standard errors
  matched_data <- data[c(m.out$index.treated, m.out$index.control), ]
  matched_data$matched_pair <- rep(1:length(m.out$index.treated), 2)
  matched_data$weights <- c(m.out$weights, m.out$weights)
  
  # Fit model with clustered SEs
  model <- feols(
    as.formula(paste0(Y, " ~ ", treat, " | ", cluster_var)),
    data = matched_data,
    weights = ~weights,
    cluster = cluster_var
  )
  
  # Extract results
  tidy_results <- tidy(model, conf.int = TRUE) |> 
    filter(term == treat)
  return(tidy_results)
}


# Urban models
mods_u_matched <- list(
  "To government" = matching_analysis(urban_cc, "compgov", "govconn", covar1, "distrid"),
  "Through government offices" = matching_analysis(urban_cc, "comp_off", "govconn", covar1, "distrid")
)

# Rural models
mods_r_matched <- list(
  "To village committee" = matching_analysis(rural_cc, "compl", "govconn", covar2, "v_id"),
  "To fellow villagers" = matching_analysis(rural_cc, "compl_vill", "govconn", covar2, "v_id")
)

# Function to prepare plotting data
prepare_plot_data <- function(model_list, area_label) {
  bind_rows(lapply(names(model_list), function(nm) {
    model_list[[nm]] |>
      mutate(outcome = nm,
             area = area_label)
  }))
}

# Combine urban and rural results
coef_df_matched <- bind_rows(
  prepare_plot_data(mods_u_matched, "Urban"),
  prepare_plot_data(mods_r_matched, "Rural")
) |>
  mutate(outcome = factor(outcome, levels = unique(outcome)),
         area = factor(area, levels = c("Urban", "Rural")))


ggplot(coef_df_matched, 
       aes(x = outcome,
           y = estimate,
           ymin = conf.low,
           ymax = conf.high)) +
  # Map linetype to area so the error‐bar shows up in the legend
  geom_errorbar(aes(linetype = area), width = 0.10, linewidth = 0.6) +
  geom_point(aes(shape = area), size = 3) +
  geom_text(aes(
    label = sprintf("%.2f", estimate),
    y = ifelse(estimate >= 0, conf.high + 0.01, conf.low - 0.01)
  ),
  size = 3, show.legend = FALSE) +
  geom_hline(yintercept = 0, linetype = "dashed") +
  facet_wrap(~ area, nrow = 1, scales = "free_x") +
  
  # Define a single legend (no title) that combines shape and linetype
  scale_shape_manual(
    name   = NULL,
    values = c(Urban = 16, Rural = 17),
    breaks = c("Urban", "Rural")
  ) +
  scale_linetype_manual(
    name   = NULL,
    values = c(Urban = "solid", Rural = "solid"),
    breaks = c("Urban", "Rural")
  ) +
  
  labs(
    y     = "Effect of political connections (Matched Dataset)",
    x     = NULL
  ) +
  theme_minimal(base_size = 12) +
  theme(
    panel.spacing = unit(1, "lines"),
    axis.text.x   = element_text(angle = 45, hjust = 1),
  )

##################################################################
## Figures 3: Sensitivity Analysis
##################################################################

par(mfrow = c(1,2))
# --- sensitivity wrapper 
run_sensitivity <- function(data, Y, tr, Covars, id_var,
                            benchmarks = Covars) {
  
  # fixed‑effect formula: RHS | FE
  rhs  <- paste(c(tr, Covars), collapse = " + ")
  fmla <- as.formula(paste0(Y, " ~ ", rhs, " | ", id_var))
  
  # fit with cluster‑robust SEs
  fit <- feols(fmla, data = data, cluster = id_var)
  
  # sensemakr
  sm <- sensemakr(
    model                = fit,
    treatment            = tr,
    benchmark_covariates = benchmarks,
    kd                   = 1          # one‑covariate benchmarks
  )
  ovb_contour_plot(sm, sensitivity.of = "t-value")
}

run_sensitivity(
  data     = rural,
  Y        = "compl",
  tr       = "govconn",
  Covars   = covar2,
  id_var   = "v_id"
) 


run_sensitivity(
  data     = urban,
  Y        = "compgov",
  tr       = "govconn",
  Covars   = covar1,
  id_var   = "distrid"
) 


